precision mediump float;		//给出默认浮点精度
varying vec2 vTextureCoord;			//接收从顶点着色器传递过来的纹理坐标
varying vec3 vPosition;  				//接收从顶点着色器传递过来的顶点位置

uniform sampler2D m_DiffuseMap;		//纹理内容数据（外观）
uniform mat4 g_WorldMatrix; 				//变换矩阵
uniform vec3 g_CameraPosition;				//摄像机位置
uniform vec3 g_LightPosition;		//光源位置

uniform float m_LShadow;
uniform float m_RShadow;
uniform float m_TShadow;
uniform float m_BShadow;
uniform float m_RectWidth;
uniform float m_RectHeight;
uniform float m_CoverAlpha;

uniform float m_DrawAlpha;
uniform bool m_EnableShadow;
uniform float m_ShadowMaxAlpha;
uniform float m_ShadowFadeOutStart;

void main(){   
   float CurAlpha = 1.0;
   vec4 finalColor=texture2D(m_DiffuseMap, vTextureCoord);        //根据纹理坐标采样出片元颜色值 
   gl_FragColor = finalColor;
   
   if(m_DrawAlpha >= 0.9){
       gl_FragColor = finalColor;
   }else{
       gl_FragColor = finalColor * m_DrawAlpha;
   }
   
   gl_FragColor.rgb = gl_FragColor.rgb * (1.0-m_CoverAlpha);

   if(m_EnableShadow){
     if(vPosition.x <= m_LShadow){
       if(vPosition.x <= m_LShadow-m_LShadow*m_ShadowFadeOutStart){
           CurAlpha = m_ShadowMaxAlpha; 	
       } else {
           CurAlpha = m_ShadowMaxAlpha - m_ShadowMaxAlpha*(vPosition.x - m_LShadow + m_LShadow*m_ShadowFadeOutStart)/(m_LShadow*m_ShadowFadeOutStart);
       }
       gl_FragColor.rgb = gl_FragColor.rgb * (1.0-CurAlpha);
     } else if(vPosition.x >= m_RShadow){
       float Sw = m_RectWidth - m_RShadow;
       if(vPosition.x >= (m_RShadow + m_ShadowFadeOutStart*Sw)){
           CurAlpha = m_ShadowMaxAlpha;
       } else {
           CurAlpha = m_ShadowMaxAlpha*(vPosition.x - m_RShadow)/(Sw*m_ShadowFadeOutStart);
       }
       gl_FragColor.rgb = gl_FragColor.rgb * (1.0-CurAlpha);
     }

     if(vPosition.y <= m_BShadow){
       if(vPosition.y <= m_BShadow-m_BShadow*m_ShadowFadeOutStart){
           CurAlpha = m_ShadowMaxAlpha;
       } else {
           CurAlpha = m_ShadowMaxAlpha - m_ShadowMaxAlpha*(vPosition.y - m_BShadow+m_BShadow*m_ShadowFadeOutStart)/(m_BShadow*m_ShadowFadeOutStart);
       }
       gl_FragColor.rgb = gl_FragColor.rgb * (1.0-CurAlpha);
     } else if(vPosition.y >= m_TShadow){
       float Sw = m_RectHeight - m_TShadow;
       if(vPosition.y >= (m_TShadow + Sw*m_ShadowFadeOutStart)){
           CurAlpha = m_ShadowMaxAlpha;
       } else {
           CurAlpha = m_ShadowMaxAlpha*(vPosition.y - m_TShadow)/(Sw*m_ShadowFadeOutStart);
       }
       gl_FragColor.rgb = gl_FragColor.rgb * (1.0-CurAlpha);
     }
   }
}